import os
import base64
import json
import time
from PIL import Image
from io import BytesIO
import logging
from openai import OpenAI

class ImageValidator:
    def __init__(self, api_key=None):
        self.client = OpenAI(
            base_url="https://ark.cn-beijing.volces.com/api/v3",
            api_key=api_key or os.environ.get("ARK_API_KEY")
        )
        if not self.client.api_key:
            raise ValueError("未找到API密钥，请设置环境变量ARK_API_KEY或通过参数传递")

        logging.basicConfig(
            format='%(asctime)s - %(levelname)s - %(message)s',
            level=logging.INFO
        )

    def _image_to_base64(self, image_path):
        try:
            with Image.open(image_path) as img:
                buffered = BytesIO()
                img.convert('RGB').save(buffered, format="JPEG")
                return base64.b64encode(buffered.getvalue()).decode('utf-8')
        except Exception as e:
            logging.error(f"图片处理失败: {str(e)}")
            return None

    def analyze_image(self, image_path):
        base64_image = self._image_to_base64(image_path)
        if not base64_image:
            return {
                'is_false_positive': False,
                'uav_detected': False, 
                'probability': 0.0,
                'reason': '图片处理失败',
                'response_time': 0.0
            }

        system_prompt = """
你是一个专业的图像分析专家，请严格按照以下要求响应：
1. 输出必须为合法的JSON格式
2. 包含三个字段：
   probability（误报概率0-1）
   is_uav（是否是无人机 bool）
   reason（分析理由）
3. 不要包含任何额外说明

请分析该监控画面：
- 判断是否为安全误报
- 识别画面中是否存在无人机
- 给出详细分析理由
"""

        try:
            start_time = time.time()
            response = self.client.chat.completions.create(
                model="doubao-vision-pro-32k-241028",
                messages=[
                    {"role": "system", "content": system_prompt},
                    {
                        "role": "user",
                        "content": [
                            {
                                "type": "image_url",
                                "image_url": {
                                    "url": f"data:image/jpeg;base64,{base64_image}"
                                }
                            },
                            {"type": "text", "text": "请分析该监控画面是否为误报"}
                        ]
                    }
                ]
            )
            end_time = time.time()
            response_time = end_time - start_time
            try:
                result = response.choices[0].message.content
                result_json = json.loads(result)
                if not all(key in result_json for key in ('probability', 'is_uav', 'reason')):
                    raise ValueError("缺少必要字段")
                
                return {
                    'is_false_positive': result_json['probability'] > 0.5,
                    'uav_detected': result_json['is_uav'],
                    'probability': result_json['probability'],
                    'reason': result_json['reason'],
                    'response_time': response_time
                }
            except (json.JSONDecodeError, ValueError) as e:
                logging.error(f"响应解析失败: {str(e)}")
                return {
                    'is_false_positive': "误报" in result,
                    'uav_detected': False,
                    'probability': 0.0,
                    'reason': '解析失败',
                    'response_time': response_time
                }

        except Exception as e:
            end_time = time.time()
            response_time = end_time - start_time
            logging.error(f"API调用失败: {str(e)}")
            return {
                'is_false_positive': False,
                'uav_detected': False,
                'probability': 0.0,
                'reason': 'API调用失败',
                'response_time': response_time
            }


if __name__ == "__main__":
    import argparse
    
    parser = argparse.ArgumentParser(description='图像误报分析工具')
    parser.add_argument('image_path', help='需要分析的图片路径')
    parser.add_argument('--api_key', default="5abbb5f5-1929-44a8-b8f2-b40fd024efc3",help='豆包API密钥（可选）')
    args = parser.parse_args()

    validator = ImageValidator(args.api_key)
    is_false_positive = validator.analyze_image(args.image_path)
    
    print(f"综合分析结果：\n误报概率: {is_false_positive['probability']:.2f}\n无人机识别: {'是' if is_false_positive['uav_detected'] else '否'}\n判定结果: {'误报' if is_false_positive['is_false_positive'] else '真实威胁'}\n响应时间: {is_false_positive['response_time']:.2f}秒\n分析理由: {is_false_positive['reason']}")